(*******************************************************************************
  This is a very simple implementation of Midas DADE
  v1.0

  This software is a freeware. Use it at your own risk.

  Installation :
  - install daMidas unit to your IDE
  - Create a 'dummy' named provider in the server
  - Create a GetTableNames procedure on the server (see below for samples)
  - Put a TDispatchConnection Decendant Component (DCOMConnection, Socket Connection, etc) on your form
  - Configure your ReportDesigner Component :
    Set SessionType property to MidasSession
    Set Databasename property to your TDispatchConnection decendant component name

  The following is a sample GetTableNames with ADO Connection Component

  procedure GetTableNames(var TableNames: OleVariant; SystemTables:WordBool); safecall;
  var TS:TStrings;
  begin
    TS:=TStringList.Create;
    try
      ADO.GetTableNames(TS,SystemTables);
      TableNames:=TS.Text;
    finally
      TS.Free;
    end;
  end;

  Any comments, suggestion, feedback please contact me :
  Hartono Setiono
  e-mail : hartono@cltek.com
  website : www.cltek.com

  under development :
  - Nested Dataset support
  - Select / use more providername
  - local sorting (take the advantage of clientdataset)
  - local dataview linking
*******************************************************************************)
unit daMidas;

interface

uses Classes, SysUtils, Forms, ExtCtrls,  DB,
     ppClass, ppComm, ppDBPipe, ppDB, ppClasUt, ppTypes,
     daDB, daQueryDataView, daDataView, daPreviewDataDlg,
     MConnect, dbclient;

type

  { TdaChildClientDataSet }
  TdaChildClientDataSet = class(TClientDataSet)
    protected

    public
      function HasParent: Boolean; override;
    end;  {class, TdaClientDataSet}

  { TdaMidasSession }
  TdaMidasSession = class(TdaSession)
    private

    protected
      function GetDefaultDatabase(const aDatabaseName: String): TComponent; override;
      function IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean; override;

    public
      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;
      class function GetDefaultMidasConnection: TDispatchConnection;

      procedure GetDatabaseNames(aList: TStrings); override;
      function  GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function  ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdaMidasSession}


  { TdaClientDataSet }
  TdaClientDataSet = class(TdaDataSet)
    private
      FQuery: TClientDataSet;
      FDispatchConnection: TDispatchConnection;

      function GetQuery: TClientDataSet;

    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabase(aDatabase: TComponent); override;
      procedure SetDataName(const aDataName: String); override;

      property Query: TClientDataSet read GetQuery;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;

      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdaClientDataSet}


  { TdaClientDataView }
  TdaClientDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildClientDataSet;

    protected
      procedure SQLChanged; override;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;

      procedure Init; override;
      procedure ConnectPipelinesToData; override;

    published
      property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdaClientDataView}


  {Delphi design time registration}
  procedure Register;


implementation

const
  cDefaultConnection = 'DefaultMidasConnection';

var
  uMidasConnection: TDispatchConnection;

{******************************************************************************
 *
 ** R E G I S T E R
 *
{******************************************************************************}

procedure Register;
begin
  {Midas DataAccess Components}
  RegisterNoIcon([TdaChildClientDataSet]);

  {Midas DataViews}
  RegisterNoIcon([TdaClientDataView]);
end;

{******************************************************************************
 *
 ** C H I L D   C L I E N T   D A T A S E T   C O M P O N  E N T S
 *
{******************************************************************************}


{------------------------------------------------------------------------------}
{ TdaChildClientDataSet.GetDefaultMidasConnection }

function TdaChildClientDataSet.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{******************************************************************************
 *
 ** M I D AS  S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaMidasSession.ClassDescription }

class function TdaMidasSession.ClassDescription: String;
begin
  Result := 'MidasSession';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaMidasSession.DataSetClass }

class function TdaMidasSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaClientDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaMidasSession.DatabaseClass }

class function TdaMidasSession.DatabaseClass: TComponentClass;
begin
  Result := TDispatchConnection;
end;

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetDefaultMidasConnection }

class function TdaMidasSession.GetDefaultMidasConnection: TDispatchConnection;
begin

  {create the default ADO Connection, if needed}
  if (uMidasConnection = nil) then
    begin
      {create default ADO Connection}
      uMidasConnection := TDispatchConnection.Create(nil);
      uMidasConnection.Name := cDefaultConnection;

    end;

  Result := uMidasConnection;

end; {function, GetDefaultADOConnection}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetTableNames }

procedure TdaMidasSession.GetTableNames(const aDatabaseName: String; aList: TStrings);
var
  lConnection: TDispatchConnection;
  TableNames:OleVariant;
begin

  {get the database}
  lConnection := TDispatchConnection(GetDatabaseForName(aDatabaseName));

  {connection must be active to get table names}
  if not lConnection.Connected then
    lConnection.Connected := True;

  TableNames:='';
  
  if lConnection.Connected then
    lConnection.AppServer.GetTableNames(TableNames, False);

  aList.Text := TableNames;

end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetDatabaseNames }

procedure TdaMidasSession.GetDatabaseNames(aList: TStrings);
begin

  {call inherited to build list of available TDispatchConnection components}
  inherited GetDatabaseNames(aList);

  {could add hard-coded connection strings here, or could
   read from an .ini file }

end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetDefaultDatabase }

function TdaMidasSession.GetDefaultDatabase(const aDatabaseName: String): TComponent;
var
  lConnection: TDispatchConnection;
begin

  lConnection := GetDefaultMidasConnection;

  {set ConnectionString property, if needed}
  if (lConnection.Name <> aDatabaseName) then
    begin
      if lConnection.Connected then
        lConnection.Connected := False;
      lConnection.Name := aDatabaseName;

    end;

  Result := lConnection;

end; {function, GetDefaultDatabase}


{------------------------------------------------------------------------------}
{ TdaMidasSession.IsNamedDatabase }

function TdaMidasSession.IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean;
begin

  Result := (AnsiCompareText(aDatabase.Name, aDatabaseName) = 0);

end; {function, IsNamedDatabase}


{------------------------------------------------------------------------------}
{ TdaMidasSession.ValidDatabaseTypes }

function TdaMidasSession.ValidDatabaseTypes: TppDatabaseTypes;
begin
  {although Sybase and Oracle ran into trouble during testing of Midas, perhaps future Midas drivers will work}
  Result := [dtMSAccess, dtMSSQLServer, dtSybaseASA, dtSybaseASE, dtOracle, dtOther];
end; {procedure, ValidDatabaseTypes}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetDatabaseType }

function TdaMidasSession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType;
begin
  {we're guessing here until we can figure out how to convert an Midas string to a database type}
  Result := dtMSAccess;
end; {function, GetDatabaseType}

{******************************************************************************
 *
 ** C L I E N T  D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaMidasSession.Create }

constructor TdaClientDataSet.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FQuery := nil;
end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaMidasSession.Destroy }

destructor TdaClientDataSet.Destroy;
begin
  FQuery.Free;
  inherited Destroy;
end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaMidasSession.ClassDescription }

class function TdaClientDataSet.ClassDescription: String;
begin
  Result := 'ClientDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetActive }

function TdaClientDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaMidasSession.SetActive }

procedure TdaClientDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetQuery }

function TdaClientDataSet.GetQuery: TClientDataSet;
begin
  if (FQuery = nil) then
    FQuery := TClientDataSet.Create(Self);
  Result := FQuery;
end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaClientDataSet.SetDatabase }

procedure TdaClientDataSet.SetDatabase(aDatabase: TComponent);
begin

  inherited SetDatabase(aDatabase);

  {table cannot be active to set database property}
  if GetQuery.Active then
    FQuery.Active := False;

  FDispatchConnection := TDispatchConnection(aDatabase);

  {set ADOQuery.Connection}
  FQuery.RemoteServer := FDispatchConnection;
  FQuery.Providername := 'Dummy';

end; {procedure, SetDatabase}

{------------------------------------------------------------------------------}
{ TdaMidasSession.SetDataName }

procedure TdaClientDataSet.SetDataName(const aDataName: String);
begin
  inherited SetDataName(aDataName);
  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;
  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.CommandText := 'SELECT * FROM ' + aDataName +
                     ' WHERE ''c'' <> ''c'' ';
end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaMidasSession.BuildFieldList }

procedure TdaClientDataSet.BuildFieldList;
var
  liIndex: Integer;
  lQueryField: TField;
  lField: TppField;
begin
  inherited BuildFieldList;

  {set table to active}
  if not(GetQuery.Active) then
    FQuery.Active := True;

  {create TppField objects for each field in the table}
  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lQueryField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName := DataName;
      lField.FieldName := lQueryField.FieldName;
      lField.FieldAlias := lQueryField.DisplayLabel;
      lField.FieldLength := lQueryField.Size;
      lField.DataType := ppConvertFieldType(lQueryField.DataType);
      lField.DisplayWidth := lQueryField.DisplayWidth;

      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetFieldNamesForSQL }

procedure TdaClientDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TClientDataSet;
begin
  aList.Clear;
  lQuery := TClientDataSet.Create(Self);
  lQuery.RemoteServer := FDispatchConnection;
  lQuery.ProviderName := 'Dummy';
  lQuery.CommandText := aSQL.Text;
  lQuery.GetFieldNames(aList);
  lQuery.Free;
end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaMidasSession.GetFieldsForSQL }

procedure TdaClientDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TClientDataSet;
  lQueryField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;
  lQuery := TClientDataSet.Create(Self);
  try
    lQuery.RemoteServer := FDispatchConnection;
    lQuery.ProviderName := 'Dummy';
    lQuery.CommandText := aSQL.Text;
    lQuery.Active := True;
    for liIndex := 0 to lQuery.FieldCount - 1 do
      begin
        lQueryField := lQuery.Fields[liIndex];
        lField := TppField.Create(nil);
        lField.FieldName := lQueryField.FieldName;
        lField.FieldAlias := lQueryField.DisplayLabel;
        lField.FieldLength := lQueryField.Size;
        lField.DataType := ppConvertFieldType(lQueryField.DataType);
        lField.DisplayWidth := lQueryField.DisplayWidth;
        aList.Add(lField);
      end;

  finally
    lQuery.Free;

  end;

end; {procedure, GetFieldsForSQL}


{******************************************************************************
 *
 ** C L I E N T D A T A S E T  Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaClientDataView.GetFieldsForSQL }

constructor TdaClientDataView.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);

  FQuery := TdaChildClientDataSet.Create(Self);

  FDataSource := TppChildDataSource.Create(Self);
  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaClientDataView.Destroy }

destructor TdaClientDataView.Destroy;
begin

  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaClientDataView.SessionClass }

class function TdaClientDataView.SessionClass: TClass;
begin
  Result := TdaMidasSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaClientDataView.PreviewFormClass }

class function TdaClientDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}

{------------------------------------------------------------------------------}
{ TdaClientDataView.ConnectPipelinesToData }

procedure TdaClientDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;

end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaClientDataView.Init }

procedure TdaClientDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;

  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaClientDataView.SQLChanged }

procedure TdaClientDataView.SQLChanged;
var
  lDatabase: TComponent;
begin

  if FQuery.Active then
    FQuery.Close;

  lDatabase := Session.GetDatabaseForName(SQL.DatabaseName);

  if (lDatabase = nil) then
    raise EDataError('TdaClientDataView.SQLChanged: No DispatchConnection object found, ' + SQL.DatabaseName);


  FQuery.RemoteServer :=  TDispatchConnection(lDatabase);
  FQuery.ProviderName := 'Dummy';
  FQuery.CommandText := SQL.MagicSQLText.Text;

end; {procedure, SQLChanged}


initialization

  {register the Midas descendant classes}
  RegisterClasses([TdaChildClientDataSet]);

  {register DADE descendant session, dataset, dataview}
  daRegisterSession(TdaMidasSession);
  daRegisterDataSet(TdaClientDataSet);
  daRegisterDataView(TdaClientDataView);

  {initialize internal reference variables}
  uMidasConnection := nil;


finalization

  {free the default connection object}
  uMidasConnection.Free;

  {unregister the Midas descendant classes}
  UnRegisterClasses([TdaChildClientDataSet]);

  {unregister DADE descendant the session, dataset, dataview}
  daUnRegisterSession(TdaMidasSession);
  daUnRegisterDataSet(TdaClientDataSet);
  daUnRegisterDataView(TdaClientDataView);
end.
